const prefix = "recce-";

export const localStorageKeys = {
  bypassSaveOverwrite: `${prefix}-bypass-save-overwrite`,
  previewChangeFeedbackID: `${prefix}-preview-change-feedback`,
  prepareEnvGuideID: `${prefix}-prepare-env`,
};
