.index-column {
  color: rgb(128, 128, 128);
  text-align: right;
}

.diff-header-added {
  /* green-700 theme.ts */
  background-color: #15803D;
  color: white;
}

.diff-cell-added {
  background-color: #cefece;
  color: black;
}

.diff-header-removed {
  /* rose-500 theme.ts */
  background-color: #F43F5E;
  color: white;
}

.diff-cell-removed {
  background-color: #ffc5c5;
  color: black;
}

.diff-cell-modified {
  background-color: #ffc5c5;
  color: black;
}

.cell-show-context-menu {
  overflow: visible;
}

.grid-header:hover .pin-icon {
  display: block;
}

.grid-header:hover .key-icon {
  display: block;
}

.rdg-cell {
  border-width: 0 1px 1px 0;
  border-style: solid;
  border-color: var(--rdg-border-color);
}

.rdg-cell[role="columnheader"] {
  /* neutral-100 themes.ts */
  background-color: #F5F5F5;
}

.rdg-row:hover .row-context-menu {
  visibility: visible;
  width: auto;
}
