const prefix = "recce";

export const sessionStorageKeys = {
  recommendationIgnored: `${prefix}-recommendation-ignored`,
  recommendationShowed: `${prefix}-recommendation-showed`,
  prevRefreshTimeStamp: `${prefix}-prev-refresh-timestamp`,
  lineageNotificationDismissed: `${prefix}-lineage-notification-dismissed`,
};
