// import { defineStyle, defineStyleConfig } from "@chakra-ui/react";
//
// // define the base component styles
// const baseStyle = {
//   borderRadius: "4px", // add a border radius
// };
//
// // export the component theme
// export const tooltipTheme = defineStyleConfig({ baseStyle });
